#include <e_maths.h>
#include <time.h>

#define KRA_VERSION "lib krand : VERSION 1.00  B. Piette 26/11/99"

#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 53774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define RNMAX (1.0-4.6e-10)

static long rand_seed1 = 0;
static long rand_seed2 = 123456789;
static long iy;
static long iv[NTAB];

/***********************************************************/
/* e_maths_seedran : set the seed for the random generator */
/* if seed <=0 : the current time is taken                 */
/***********************************************************/
void e_maths_seedran(long seed)
{ int j;
  long k;

  /* by default we take the current time as the seed */
  if (seed  < 1) rand_seed1 = (long )time(0);
  else rand_seed1 = seed;

  rand_seed2 = rand_seed1;

  for(j = NTAB+7; j >= 0; j--)
  { k = rand_seed1/IQ1;
    rand_seed1 = IA1*(rand_seed1-k*IQ1)-k*IR1;
    if (seed < 0) rand_seed1 += IM1;
    if (j < NTAB) iv[j] = rand_seed1;    
  }
  iy = iv[0];
}

/**********************************************************/
/* Return a randum number in the range ]0, 1[ as a double */
/* The period is > 2e18 (cfr Numerical Recipies )         */
/**********************************************************/
double e_maths_dran()
{ int j;
  long k;
  double temp;
  
  if(rand_seed1 < 1) e_maths_seedran(0);
  
  k = rand_seed1/IQ1;
  rand_seed1 = IA1*(rand_seed1-k*IQ1)-k*IR1;
  if (rand_seed1 < 0) rand_seed1 += IM1;
  k = rand_seed2/IQ2;
  rand_seed2 = IA2*(rand_seed2-k*IQ2)-k*IR2;
  if (rand_seed2 < 0) rand_seed2 += IM1;
  
  j = iy/NDIV;
  iy = iv[j]-rand_seed2;
  iv[j] = rand_seed1;
  if(iy < 1) iy += IMM1;
  if((temp=AM*iy) > RNMAX)  return(RNMAX);
  
  return (temp);
}
