#include <e_test.h>

/******************************************************/
/* A set of test function of the type                 */
/* test(obj *op,void *val)                            */
/*    op must be an object to test                    */
/*    val : a value to test. It can be ignored        */
/*          but if non null it should be an object    */
/*    the link count must never change                */
/* These function are typically used by lib functions */
/* for general tests procedures.                      */
/* RETURN : 1 : test is positive.                     */
/*          0 : test failed or error                  */
/* LINK : NO CHANGE                                   */
/******************************************************/

/****************************** TESTS OBJECT TYPES ***********************/
/**************************/
/* test if op is a string */
/**************************/
int e_test_is_string(e_obj *op,void *val)
{ if(!op) return(0);
  return(E_IS_STRING(op));
}

/****************************** STRING TESTS ***********************/
/********************************/
/* string comparison : op < val */
/********************************/
int e_test_string_less(e_obj *op,void *val)
{ if(!op || !E_IS_STRING(op) || !val || !E_IS_STRING(val)) return(0);
  
  return((strcmp(E_STRING_S(op),E_STRING_S(val)) < 0) ? 1 : 0);
}

/*********************************/
/* string comparison : op <= val */
/*********************************/
int e_test_string_less_or_equal(e_obj *op,void *val)
{ if(!op || !E_IS_STRING(op) || !val || !E_IS_STRING(val)) return(0);
  
  return((strcmp(E_STRING_S(op),E_STRING_S(val)) <= 0) ? 1 : 0);
}

/********************************/
/* string comparison : op > val */
/********************************/
int e_test_string_greater(e_obj *op,void *val)
{ if(!op || !E_IS_STRING(op) || !val || !E_IS_STRING(val)) return(0);
  
  return((strcmp(E_STRING_S(op),E_STRING_S(val)) > 0) ? 1 : 0);
}

/********************************/
/* string comparison : op >= val */
/********************************/
int e_test_string_greater_or_equal(e_obj *op,void *val)
{ if(!op || !E_IS_STRING(op) || !val || !E_IS_STRING(val)) return(0);
  
  return((strcmp(E_STRING_S(op),E_STRING_S(val)) >= 0) ? 1 : 0);
}

/********************************/
/* op match unix pattern val    */
/********************************/
int e_test_string_unix_match(e_obj *op,void *val)
{ if(!op || !E_IS_STRING(op) || !val || !E_IS_STRING(val)) return(0);
  
  return(e_tools_unix_match(E_STRING_S(val),E_STRING_S(op)));
}
