#include <sysf_init.h>
#include <sysf_com.h>
#include <sysf_dir.h>
#include <ctype.h>
#include <unistd.h>
#include <sysf.h>
#include <sysf.BUILD>


/**************************************************************/ 
/* sysf : Unix/Linux system file management system            */
/**************************************************************/ 

char VERSION[] = "sysf: v2.04."BUILD_NO"; "BUILD_DATE";  Bernard Piette";

char HostName[E_LINE_SIZE];

/* Define some Text Constants */
char *Help[] = { 
"Syntax : sysf [PARAMETERS] [COMMAND] [HOSTS] [ENTRIES]",
" PARAMETERS:",
"  -echo=e    : echo host names and config files as they are being treated.",
"  -echo_host=eh  : echo host names as they are being treated.",
"  -mailto=m2 EMAIL_ADDRESS : Email the output to the specified address", 
"  -config_dir=cd CONFIG_DIR : config directory path.",
"  -config_file=cf CONFIG_FILE : config file name [sysf.conf].",
"  -keep_dead_hosts=kdh : do not remove the hosts from the DEAD group",
"  -by_system_file=bsf : select entries by their system file.",
"  -rsh : use rsh for remote access."
"  -ssh : use ssh for remote access."
"  -forgiving : does not abort when some errors occurs",
"  -exclude_entry_names=xen : coma separated list of entry names to exclude",
" COMMANDS:",
"  -install=I : install the config file (config.* -> sys)",
"  -first_install=FI : as install but make a .DIST copy of the system file.",
"  -OS_install=OSI : ignore SHARED files, first_install for all file but",
"             -new on files with h flag set when .HOST file does not exist.",
"               and create a sys.DIST file if it does not exist.",
"  -info=i    : display information about the config file",
"  -archive=A : archive the system file (sys -> config.*) if the system", 
"               and the config file differ. \".HOST\" config file only.", 
"  -new=N     : create a new config file (sys -> config.HOST)",
"  -makedef=MD : create or update the DEFAULT configuration file.",
"  -makeshared=MS : create or update the SHARED configuration file.",
"  -makegroup=MG HOST_GROUP : create or update the HOST_GROUP configuration file.",
"  -uninstall=U : Execute the config command with the \"-Remove\" flag.",
"  -install_mode=Im, -install_uid=Iu, -install_gid=-Ig : copy file mode,",
"            uid and gid from config to system file", 
"  -archive_mode=Am, -archive_uid=Au, -archive_gid=-Ag : copy file mode,",
"            uid and gid from system to config file", 
"  -check=c   : check if the config file match the system file.",
"  -is_not_conf=nc : lists sys entries with no system or no config file",
"  -diff=d    : display the differences between the config and system files",
"  -diff_host=dh HOST : display the differences between the config files",
"               for this host and those listed in HOSTS.",
"  -check_host=ch HOST : check that the config files for this host and those",
"               listed in HOSTS are identical.",
"  -cat=C     : display the config file",
"  -catsys=Cs : display the system file",
"  -list_groups=lg : list the config groups",
"  -list_configs=lc : list all the config files by groups",
"  -list_1group=l1g GROUP: list the config files in that group",
"  -list_hosts=lh : list the host groups with their members",
"  -list_host_group=lhg HOST_GROUP : list the host members of the specified ",
"                host group.",
"  -list_these_hosts_group=lthg : list the groups the hosts belongs to ",
"  -find_group=fg : list the group each entry belong to",
"  -list_arg_hosts=lah : list the argument hosts : hosts - exclude_hosts",
"  -list_arg_entry_names=laen : list the argument entry names : "
"                         ENTRIES - exclude_entry_names",
" HOSTS:",
"  -hosts=h HOST1,HOST2,HOST_GROUP...",
"  -exclude_hosts=xh HOST1,HOST2, ...",
" ENTRIES:",
"  ENTRY_NAMES : list of config names",
"  DIR_GROUPS : all config names in the specified group.",
"  ALL        : all the config names",
0};

struct {
  int install;              
  int first_install; 
  int OS_install;             
  int info;
  int archive;
  int new;
  int makedef;
  int makeshared;
  char makegroup[E_LINE_SIZE];
  int uninstall;              
  int install_mode;              
  int install_uid;              
  int install_gid;              
  int archive_mode;              
  int archive_uid;              
  int archive_gid;              
  int check;
  int is_not_conf;
  int diff;
  char check_host[E_LINE_SIZE];
  char diff_host[E_LINE_SIZE];
  int cat;
  int catsys;
  int list_groups;
  int list_configs;
  char list_1group[E_LONG_LINE_SIZE];
  int list_hosts;
  int list_arg_hosts;
  int list_arg_entry_names;
  char list_host_group[E_LONG_LINE_SIZE];
  int list_these_hosts_groups;
  int find_group;
  char hosts[E_LONG_BUFF_SIZE];
  char exclude_hosts[E_LONG_BUFF_SIZE];
  char exclude_entry_names[E_LONG_BUFF_SIZE];
  char mailto[E_LINE_SIZE];
  char config_dir[E_LONG_BUFF_SIZE];
  char config_file[E_LINE_SIZE];
  int echo;
  int echo_host;
  int keep_dead_hosts;
  int rsh;
  int ssh;
  int forgiving;
  int by_system_file;
} par;

struct sysf_par sysfile;
sysf_cf sysf_cf_vars;
int Forgiving;

e_parameter Pars[] = {
{ "install",E_CTYPE_FLAG,&par.install,0,1,"I",0},   
{ "first_install",E_CTYPE_FLAG,&par.first_install,0,1,"FI",0},   
{ "OS_install",E_CTYPE_FLAG,&par.OS_install,0,1,"OSI",0},   
{ "info",E_CTYPE_FLAG,&par.info,0,1,"i",0},   
{ "archive",E_CTYPE_FLAG,&par.archive,0,1,"A",0},   
{ "new",E_CTYPE_FLAG,&par.new,0,1,"N",0},   
{ "makedef",E_CTYPE_FLAG,&par.makedef,0,1,"MD",0},   
{ "makeshared",E_CTYPE_FLAG,&par.makeshared,0,1,"MS",0},   
{ "makegroup",E_CTYPE_STRING,par.makegroup,"",0,"MG",0},   
{ "uninstall",E_CTYPE_FLAG,&par.uninstall,0,1,"U",0},   
{ "install_mode",E_CTYPE_FLAG,&par.install_mode,0,1,"Im",0},   
{ "install_uid",E_CTYPE_FLAG,&par.install_uid,0,1,"Iu",0},   
{ "install_gid",E_CTYPE_FLAG,&par.install_gid,0,1,"Ig",0},   
{ "archive_mode",E_CTYPE_FLAG,&par.archive_mode,0,1,"Am",0},   
{ "archive_uid",E_CTYPE_FLAG,&par.archive_uid,0,1,"Au",0},   
{ "archive_gid",E_CTYPE_FLAG,&par.archive_gid,0,1,"Ag",0},   
{ "check",E_CTYPE_FLAG,&par.check,0,1,"c",0},   
{ "is_not_conf",E_CTYPE_FLAG,&par.is_not_conf,0,1,"nc",0},   
{ "diff",E_CTYPE_FLAG,&par.diff,0,1,"d",0},   
{ "check_host",E_CTYPE_STRING,par.check_host,"",0,"ch",0},   
{ "diff_host",E_CTYPE_STRING,par.diff_host,"",0,"dh",0},   
{ "cat",E_CTYPE_FLAG,&par.cat,0,1,"C",0},   
{ "catsys",E_CTYPE_FLAG,&par.catsys,0,1,"Cs",0},   
{ "list_groups",E_CTYPE_FLAG,&par.list_groups,0,1,"lg",0},   
{ "list_configs",E_CTYPE_FLAG,&par.list_configs,0,1,"lc",0},   
{ "list_1group",E_CTYPE_STRING,par.list_1group,"",0,"l1g",0},   
{ "list_hosts",E_CTYPE_FLAG,&par.list_hosts,0,1,"lh",0},   
{ "list_arg_hosts",E_CTYPE_FLAG,&par.list_arg_hosts,0,1,"lah",0},   
{ "list_arg_entry_names",E_CTYPE_FLAG,&par.list_arg_entry_names,0,1,"laen",0},
{ "list_host_group",E_CTYPE_STRING,par.list_host_group,"",0,"lhg",0},   
{ "list_these_hosts_groups",E_CTYPE_FLAG,&par.list_these_hosts_groups,0,1,
    "lthg",0},   
{ "find_group",E_CTYPE_FLAG,&par.find_group,0,1,"fg",0},   
{ "hosts",E_CTYPE_STRING,par.hosts,"",0,"h",0},   
{ "exclude_hosts",E_CTYPE_STRING,par.exclude_hosts,"",0,"xh",0},   
{ "exclude_entry_names",E_CTYPE_STRING,par.exclude_entry_names,"",0,"xen",0},
{ "mailto",E_CTYPE_STRING,par.mailto,"",0,"m2",0},   
{ "config_dir",E_CTYPE_STRING,par.config_dir,"",1,"cd",0},   
{ "config_file",E_CTYPE_STRING,par.config_file,"",1,"cf",0},   
{ "echo",E_CTYPE_FLAG,&par.echo,0,1,"e",0},   
{ "echo_host",E_CTYPE_FLAG,&par.echo_host,0,1,"eh",0},   
{ "keep_dead_hosts",E_CTYPE_FLAG,&par.keep_dead_hosts,0,1,"kdh",0},   
{ "rsh",E_CTYPE_FLAG,&par.rsh,0,1,0,0},   
{ "ssh",E_CTYPE_FLAG,&par.ssh,0,1,0,0},   
{ "forgiving",E_CTYPE_FLAG,&par.forgiving,0,1,0,0},   
{ "by_system_file",E_CTYPE_FLAG,&par.by_system_file,0,1,"bsf",0},   
{ 0,0,0,0,0,0},
};

int ExecSysf(int Argc, char **Argv,char *HostName);
int commands_no();
int is_a_modification_commands();

/**************************************************************************/
/*                        MAIN                                            */   
/**************************************************************************/
int main(argc, argv)
int argc;
char *argv[];
{ int i,n,Argc,err,flag;
  char **Argv;
  e_string *sp,*arg_sp = 0,*spcom = 0;

  sysfile.messages = 0;
  sysfile.hosts_groups = 0;
  sysfile.CheckCounter = 0;
  sysfile.this_host_groups = 0;

  E_NULL_ERROR(e_init());
  init_dir_entry_names();

           /* Reconstruct the argument list but without the hosts */
           /*******************************************************/
  arg_sp = e_string_new(0,0,E_LINE_SIZE);
  i = 1;
  while (i < argc)
  { if(!strcmp(argv[i],"-h") || !strcmp(argv[i],"-hosts") ||
       !strcmp(argv[i],"-xh") || !strcmp(argv[i],"-exclude_hosts") ||
       !strcmp(argv[i],"-m2") || !strcmp(argv[i],"-mailto") 
      )
    { /* skip [-hosts=h] HOST_NAME        */
      /*      [-exclude_hosts=xh] NAME    */
      /*      [-mailto=m2] EMAIL_ADDRESS  */
      i++;   
    }
    else
    { arg_sp = e_string_append_buffer(arg_sp,argv[i]);
      arg_sp = e_string_append_buffer(arg_sp," ");
    }
    i++;
  }

           /* Read the argument list*/
           /*************************/
  if((Argc = e_parameter_read_arg_line(argc,argv,Pars,Help,&Argv,
                                       E_PARAMETER_V_IS_VERSION))<0)
  { goto error;
  }

           /* Default command : info */
           /**************************/
  n = commands_no();
  if(n > 1)
  { mprintf("Only one sysf command can be selected at a time.!\n");
    goto error;    
  }
  else if (n == 0)
  { par.info = 1; 
  }

           /* Set global variable to test forgiving flag */
  Forgiving = par.forgiving;

  if(gethostname(HostName,E_LINE_SIZE))
  { mprintf("Unable to get host name!\n");
    if(sysfile.messages) mail(par.mailto,"sysf");
    goto error;
  }

  if(!par.hosts[0])       /* By default use the local host */
  { strncpy(par.hosts,HostName,E_LINE_SIZE-1);
    par.hosts[E_LINE_SIZE] = 0;
  } 

           /* Set GLOBAL PARAMETERS            */
           /* DEFAULT, /etc/sysf.cf, parameter */
           /************************************/
     /* default values */
  strncpy(sysf_cf_vars.config_dir,SYSF_CONFIG_DIR,E_BUFF_SIZE);
  strncpy(sysf_cf_vars.config_file,SYSF_CONFIG_FILE,E_BUFF_SIZE);
  strncpy(sysf_cf_vars.rsh_com,RSH_COM,E_BUFF_SIZE);
  strncpy(sysf_cf_vars.ssh_com,SSH_COM,E_BUFF_SIZE);
  strncpy(sysf_cf_vars.mail_com,MAIL_COM,E_BUFF_SIZE);
  strncpy(sysf_cf_vars.remote_shell,SYSF_DEFAULT_RSHELL,E_BUFF_SIZE);
  strncpy(sysf_cf_vars.check_sum,SYSF_DEFAULT_CHECKSUM,E_BUFF_SIZE);

     /* /etc/sysf.cf values */
  E_NULL_ERROR(read_etc_sysf_cf(&sysf_cf_vars));

     /* par values */
  if(par.config_dir[0])
  { strncpy(sysf_cf_vars.config_dir,par.config_dir,E_BUFF_SIZE);
  }
  if(par.config_file[0])
  { strncpy(sysf_cf_vars.config_file,par.config_file,E_BUFF_SIZE);
  }
  if(par.rsh)
  { strncpy(sysf_cf_vars.remote_shell,"rsh",E_BUFF_SIZE);
  }
  if(par.ssh)
  { strncpy(sysf_cf_vars.remote_shell,"ssh",E_BUFF_SIZE);
  }

  E_NULL_ERROR(check_sysf_cf_pars(&sysf_cf_vars));

           /* Do we email the output?      */
           /********************************/
  if(par.mailto[0])
  { E_NULL_ERROR(sysfile.messages = e_string_new(0,"",E_LONG_BUFF_SIZE)); 
  }


           /* Read the configuration files */
           /********************************/
  E_NULL_ERROR(sysfile.configs = 
           read_sysf_config(sysf_cf_vars.config_dir,sysf_cf_vars.config_file));

  E_NULL_ERROR(make_config_group_lists());

          /* Check that config group names differ from entry names */
  E_NULL_ERROR(check_entries_and_config_groups(sysfile.configs,
                                               sysfile.config_groups));

           /* Check that all the config dir exist */
  E_NULL_ERROR(check_config_groups(sysfile.config_groups,
                                   sysf_cf_vars.config_dir));

  E_NULL_ERROR(read_sysf_hosts(sysf_cf_vars.config_dir,"hosts.conf"));


            /* Execute the info commands, no HOSTS nor CONFIG required */ 
            /***********************************************************/
  if(par.list_groups)    { display_config_groups();            goto end; }
  if(par.list_configs)   { display_all_config_files();         goto end; }
  if(par.list_1group[0]) { display_one_group(par.list_1group); goto end; }   
  if(par.list_hosts)     { display_hosts_groups();             goto end; }
  if(par.list_host_group[0]) 
                         { display_host_group(par.list_host_group); 
                           goto end;
                         }

  if(Argc < 0)           { e_Syntax(Help);                     exit(0);  }

            /* Create the list of argument hosts, resolving hosts groups */ 
            /*************************************************************/
  flag = par.keep_dead_hosts ? 0 : FLAG_REM_DEAD;
  E_NULL_ERROR(sysfile.hosts = read_args_hosts(par.hosts,flag));
  E_NULL_ERROR(sysfile.hosts =
  	           remove_args_hosts(sysfile.hosts,par.exclude_hosts));
  if(par.list_arg_hosts)     { display_arg_hosts();             goto end; }

  if(par.list_these_hosts_groups) 
  { display_these_hosts_groups(sysfile.hosts); 
    goto end; 
  }

            /* Scan the list of hosts.                                 */ 
            /* For other hosts, execute sysf remotely.                 */ 
            /***********************************************************/
  e_aset_next(sysfile.hosts,E_NEXT_RESET,0);
  while(e_aset_next(sysfile.hosts,E_NEXT_GET_NEXT,(void **)&sp) > 0)
  { 
    if(!strcmp(E_STRING_S(sp),HostName))
    { if(sysfile.this_host_groups) E_FREE(sysfile.this_host_groups);
      E_NULL_ERROR(sysfile.this_host_groups = get_this_host_groups(sp));
      E_NULL_ERROR(ExecSysf(Argc,Argv,HostName));
    }
    else if(!strcmp(sysf_cf_vars.remote_shell,"ssh")) 
    {                          /* rsh HOST "sysf ARGLIST " */
      E_NULL_ERROR(spcom = e_string_new(0,sysf_cf_vars.ssh_com,100));
      E_NULL_ERROR(spcom = e_string_append_buffer(spcom," "));
      E_NULL_ERROR(spcom = e_string_append_buffer(spcom,E_STRING_S(sp)));
      E_NULL_ERROR(spcom = e_string_append_buffer(spcom," \"sysf "));
      E_NULL_ERROR(spcom = e_string_append_buffer(spcom,E_STRING_S(arg_sp)));
      E_NULL_ERROR(spcom = e_string_append_buffer(spcom," \""));

      if(!(spcom = e_exec_sh_command(E_STRING_S(spcom),&err))) goto error;
      mprintf("%s",E_STRING_S(spcom));
    }   
    else if(!strcmp(sysf_cf_vars.remote_shell,"rsh")) 
    {                          /* rsh HOST "sysf ARGLIST " */
      E_NULL_ERROR(spcom = e_string_new(0,sysf_cf_vars.rsh_com,100));
      E_NULL_ERROR(spcom = e_string_append_buffer(spcom," "));
      E_NULL_ERROR(spcom = e_string_append_buffer(spcom,E_STRING_S(sp)));
      E_NULL_ERROR(spcom = e_string_append_buffer(spcom," \"sysf "));
      E_NULL_ERROR(spcom = e_string_append_buffer(spcom,E_STRING_S(arg_sp)));
      E_NULL_ERROR(spcom = e_string_append_buffer(spcom," \""));

      if(!(spcom = e_exec_sh_command(E_STRING_S(spcom),&err))) goto error;
      mprintf("%s",E_STRING_S(spcom));
    }   
    else if(!strcmp(sysf_cf_vars.remote_shell,"none")) 
    { mprintf("WARNING: no remote shell :\n"
              "         Can't execute sysf command on remote host %s\n",
              E_STRING_S(sp));
    }
    else /* should never happen */ 
    { mprintf("ERROR: invalid remote shell: %s\n",sysf_cf_vars.remote_shell);
    }
    E_FREE(sp);
  }
end:
  if(spcom) E_FREE(spcom);
  if(arg_sp) E_FREE(arg_sp);

  if(par.mailto[0] && sysfile.messages) mail(par.mailto,"sysf");
 
  exit(0);

error:
  if(spcom) E_FREE(spcom);
  if(arg_sp) E_FREE(arg_sp);
  

  if(par.mailto[0] && sysfile.messages)
  { sysfile.messages = e_string_append_string(sysfile.messages,
                                            e_error_2_string(E_ERROR_ANY)); 
    mail(par.mailto,"sysf");
  }

                    /* Still echo error on stderr, even if mailed!  */
  e_error_flush(E_ERROR_ANY);

  exit(1);   
}

/************************************************************************/
/* Determines which config file must be used, initialise its parameters */
/* including its size, ownership and hash code.                         */
/* Return : 1 : OK                                                      */
/*          0 : error                                                   */
/************************************************************************/
int what_config_file(char *conf,char *HostName,sysfile_info *sfp,
                     e_aset *conf_data)
{ int NoCheck;

  NoCheck = par.new;

  if(sfp->flags & E_SYSF_FLAG_s)   
  {                      /* config file with GLOBAL suffix (no check) */
    E_NULL_ERROR(get_config_file(conf,sysf_cf_vars.config_dir,conf_data,
                                 &sfp->config_file,"SHARED",1,sfp->flags));
  }
  else if(par.makedef)   
  {                      /* config file with DEFAULT suffix (no check) */
    E_NULL_ERROR(get_config_file(conf,sysf_cf_vars.config_dir,conf_data,
                                 &sfp->config_file,"DEFAULT",1,sfp->flags));
  }
  else if(par.makeshared)   
  {                      /* config file with SHARED suffix (no check) */
    E_NULL_ERROR(get_config_file(conf,sysf_cf_vars.config_dir,conf_data,
                                 &sfp->config_file,"SHARED",1,sfp->flags));
  }
  else if(par.makegroup[0]) 
  {                      /* config file with HOST GROUP suffix */
    if(e_aset_get_key_index(sysfile.hosts_groups,
                            e_name_new(par.makegroup),0) < 0 )
    { e_error(E_ERROR_BAD_ARGUMENT,E_ERROR_LEVEL_USER,
             "Host Group %s does not exist",par.makegroup);
         goto error;
    }

    E_NULL_ERROR(get_config_file(conf,sysf_cf_vars.config_dir,conf_data,
                               &sfp->config_file,par.makegroup,1,sfp->flags));
  }
  else
  {                      /* config file with HOST suffix (check) */
    get_config_file(conf,sysf_cf_vars.config_dir,conf_data,
                    &sfp->config_file,HostName,NoCheck,sfp->flags);
  }

  return(1);

error :
  return(0);
}

/*****************************/
/* Execute the sysf commands */
/* Return 1 : OK             */
/*        0 : error          */
/*****************************/
int ExecSysf(int Argc, char **Argv,char *HostName)
{ char *conf=0,ComName[E_LINE_SIZE];
  int i,NoCheck,No;
  e_aset *stp = 0,*conf_data,*entry_name_asp,*enasp;
  e_name *np;
  sysfile_info sysfip;
  file_info host_fi;

  NoCheck = par.new;               /* Don't check if config file exists  */
  get_command_name(ComName);

  if(par.echo || par.echo_host) 
  { mprintf("%s\n",HostName);
  }

     /* create an aset with all the argument config files                */
     /* sysfile.arg_configs : aset of e_name : one for each config file  */
     /* Config groups are translated into config file names              */
  E_NULL_ERROR(sysfile.arg_configs = e_aset_new(0));

  if(par.by_system_file)
  { No = 0;
         /* Create entry_name list from the system files */
    for (i=0; i < Argc; i++)
    { if((entry_name_asp = 
             get_entry_name_for_system_file(sysfile.configs,Argv[i])))
      { E_NULL_ERROR(sysfile.arg_configs = 
                  e_aset_add_aset(sysfile.arg_configs,entry_name_asp,
                                  E_ASET_ADD_MODE_UNIQUE_VAL));
        No++;
      }
      else
      { mprintf("No sysf entry for system file %s\n",Argv[i]);
      }
    }
    if(!No) 
    { mprintf("sysf ERROR: no valid entry!\n");
      return(1);
    }
  }
  else if(Argc == 0) /* No config name : use ALL */
  {             /* Add the set of names to arg_configs */

    if(is_a_modification_commands())
    { e_error(E_ERROR_BAD_ARGUMENT,E_ERROR_LEVEL_USER,
               "ERROR : sysf : no entry name specified!");

      goto error2;
    }
 
               /* make aset { "ALL" } */
    E_NULL_ERROR(enasp = e_aset_add_item(e_aset_new(0),0,
                                         e_string_new(0,"ALL",0),
                                         E_ASET_ADD_MODE_UNIQUE_VAL)); 

               /* make the entry names list */
    E_NULL_ERROR(sysfile.arg_configs = 
		 make_entry_name_aset(enasp,"entry names"));
    enasp = 0;
  }
  else
  {            /* convert argument list to aset of strings */
    E_NULL_ERROR(enasp = e_aset_new(0));
    for (i=0; i < Argc; i++)
    { E_NULL_ERROR(enasp = e_aset_add_item(enasp,0,e_string_new(0,Argv[i],0),
                                           E_ASET_ADD_MODE_UNIQUE_VAL)); 
    }

               /* Make the entry name list */
    E_NULL_ERROR(sysfile.arg_configs = 
		 make_entry_name_aset(enasp,"entry names"));
    enasp = 0;
  }

  if(par.exclude_entry_names[0])
  {        /* convert coma separated list to an aset of strings */
    E_NULL_ERROR(enasp = 
	       e_aset_split(e_string_new(0,par.exclude_entry_names,0),",",0));

           /* build the list of entry names */
    E_NULL_ERROR(enasp = make_entry_name_aset(enasp,"-exclude_entry_names"));
 
          /* remove exclude entry name list from the config argument list */
    E_NULL_ERROR(sysfile.arg_configs = 
		 e_aset_diff_by_value(sysfile.arg_configs,enasp,-1));
    E_FREE(enasp);
  }

  if(par.list_arg_entry_names) 
  { display_arg_entry_names(sysfile.arg_configs);             
    goto end; 
  }

  if(par.find_group) 
  { find_group();  
    return(1);
  }

  e_aset_next(sysfile.arg_configs,E_NEXT_RESET,0);
  while(e_aset_next(sysfile.arg_configs,E_NEXT_GET_NEXT,(void **)&np) > 0)
  { conf = E_NAME_NAME(np);

    if(par.info) mprintf("%s:\n",conf);
    else if(par.echo) 
    { if(par.makegroup[0]) 
           mprintf("%s %s %s\n",ComName,par.makegroup,conf);
      else mprintf("%s %s\n",ComName,conf);
    };

                      /* Get the data for the config file. */
    E_NULL_ERROR(conf_data = e_aset_get_value_by_key(sysfile.configs,
                                                     E_USE(np),0));

                           /* get system flags  */
    E_NULL_ERROR(get_system_file_flags(conf,conf_data,&sysfip.flags));

    E_NULL_ERROR(get_system_file(conf,conf_data,&sysfip.sys_file,
                 sysfip.flags));

    E_NULL_ERROR(get_auxiliary_command(conf,sysf_cf_vars.config_dir,conf_data,
                                    &sysfip.aux_com));    
#ifdef ZXZXZX
    if(sysfip.flags & E_SYSF_FLAG_s)   
    {                      /* config file with GLOBAL suffix (no check) */
      E_NULL_ERROR(get_config_file(conf,sysf_cf_vars.config_dir,conf_data,
                               &sysfip.config_file,"SHARED",1,sysfip.flags));
    }
    else if(par.makedef)   
    {                      /* config file with DEFAULT suffix (no check) */
      E_NULL_ERROR(get_config_file(conf,sysf_cf_vars.config_dir,conf_data,
                               &sysfip.config_file,"DEFAULT",1,sysfip.flags));
    }
    else if(par.makeshared)   
    {                      /* config file with SHARED suffix (no check) */
      E_NULL_ERROR(get_config_file(conf,sysf_cf_vars.config_dir,conf_data,
                                &sysfip.config_file,"SHARED",1,sysfip.flags));
    }
    else if(par.makegroup[0]) 
    {                      /* config file with HOST GROUP suffix */
      if(e_aset_get_key_index(sysfile.hosts_groups,
                              e_name_new(par.makegroup),0) < 0 )
      { e_error(E_ERROR_BAD_ARGUMENT,E_ERROR_LEVEL_USER,
              "Host Group %s does not exist",par.makegroup);
        goto error;
      }

      E_NULL_ERROR(get_config_file(conf,sysf_cf_vars.config_dir,conf_data,
                           &sysfip.config_file,par.makegroup,1,sysfip.flags));
    }
    else
    {                      /* config file with HOST suffix (check) */
      get_config_file(conf,sysf_cf_vars.config_dir,conf_data,
		      &sysfip.config_file,HostName,NoCheck,sysfip.flags);
    }
#endif

    E_NULL_ERROR(what_config_file(conf,HostName,&sysfip,conf_data));

    if ((sysfip.flags & E_SYSF_FLAG_C))
    {        /* W Info config file does not exist  */
             /* create it with the host extension. */
      if ((sysfip.flags & E_SYSF_FLAG_W) && !sysfip.config_file.exists)
      { get_config_file(conf,sysf_cf_vars.config_dir,conf_data,
		        &sysfip.config_file,HostName,1,sysfip.flags);
      }

                     /* Create the system file */
      if((!execute_aux_com("-CreateSysFile",ComName,&sysfip,par.echo))) 
          goto error;

                           /* check that the system file has been created */
      E_NULL_ERROR(get_system_file(conf,conf_data,&sysfip.sys_file,
                   sysfip.flags));

      if ((sysfip.flags & E_SYSF_FLAG_W))
      { E_NULL_ERROR(what_config_file(conf,HostName,&sysfip,conf_data));
      }
    }

    if(par.is_not_conf) 
    { E_NULL_ERROR(is_not_conf(conf,&sysfip,par.echo));      
    }; 

                           /* Dir lists : read system and config data */
    if((sysfip.flags & E_SYSF_FLAG_g))
    { if(sysfip.config_file.exists)
      { E_NULL_ERROR(sysfip.dir_config_asp = 
           dir_read_config(sysfip.config_file.FileName));
      }
      else sysfip.dir_config_asp = 0;

      if(sysfip.sys_file.exists)
      { E_NULL_ERROR(sysfip.dir_sys_asp = 
           dir_read_sys(sysfip.sys_file.FileName,sysfip.dir_config_asp));
      }
      else sysfip.dir_sys_asp = 0;

      if (sysfip.dir_sys_asp && sysfip.dir_config_asp)
      { sysfip.diff_asp = 
           dir_list_diff(sysfip.dir_sys_asp, sysfip.dir_config_asp);
      }
      else sysfip.diff_asp = 0;
 
    }


                           /* Check or compare two hosts */
    if(par.check_host[0]) 
    { if(!(sysfip.flags & E_SYSF_FLAG_h) && !(sysfip.flags & E_SYSF_FLAG_s))
      { get_config_file(conf,sysf_cf_vars.config_dir,conf_data,&host_fi,
                        par.check_host,0,sysfip.flags);

         E_NULL_ERROR(check_host(conf,&host_fi,par.check_host,&sysfip,HostName,
                                 par.echo));
      }
      continue;
    }; 

    if(par.diff_host[0]) 
    { if(!(sysfip.flags & E_SYSF_FLAG_s))
      { get_config_file(conf,sysf_cf_vars.config_dir,conf_data,&host_fi,
                        par.diff_host,0,sysfip.flags);
        E_NULL_ERROR(diff_host(conf,&host_fi,par.diff_host,&sysfip,HostName,
                               par.echo));
      }
      continue;
    }; 

                      /* test if the files are identical */
    test_sysfile(&sysfip);

    if(par.install || par.first_install || 
       (par.OS_install && !(sysfip.flags & E_SYSF_FLAG_s))) 
    { if((sysfip.flags & E_SYSF_FLAG_I) && !(sysfip.flags & E_SYSF_FLAG_g) &&
	(!execute_aux_com("-PreInstall","-install",&sysfip,par.echo))) 
          goto error;
        
      if(par.install)
      { E_NULL_ERROR(install(conf,&sysfip,par.echo)); 
      } 
      else if(par.first_install) 
      { E_NULL_ERROR(first_install(conf,&sysfip,par.echo));
      }
      else
      { /* OS_Install */
                /* Ignore shared files */
        if(!(sysfip.flags & E_SYSF_FLAG_s))
        {       /* by default do a first install if the config file exists */
          if(sysfip.config_file.exists)
          { first_install(conf,&sysfip,par.echo);
          }
          else
	  { if(sysfip.flags & E_SYSF_FLAG_h)
            {   /* no config file for h file: create it with new */
                /* Create the config file CONF.HOSTMAME          */
              get_config_file(conf,sysf_cf_vars.config_dir,conf_data,
		              &sysfip.config_file,HostName,1,sysfip.flags);

             if ((sysfip.flags & E_SYSF_FLAG_C))
             {        /* Create the system file */
               if(!execute_aux_com("-CreateSysFile",ComName,&sysfip,par.echo)) 
                   goto error;
                      /* check that the system file has been created */
               E_NULL_ERROR(get_system_file(conf,conf_data,&sysfip.sys_file,
                            sysfip.flags));
              }

              new(conf,&sysfip,par.echo);
	    }
          }
        }
      }

      if((sysfip.flags & E_SYSF_FLAG_C) && !(sysfip.flags & E_SYSF_FLAG_g) &&
        (!execute_aux_com("-UpdateSysFile","-install",&sysfip,par.echo))) 
           goto error;

      if((sysfip.flags & E_SYSF_FLAG_I) && !(sysfip.flags & E_SYSF_FLAG_g) &&
        (!execute_aux_com("-PostInstall","-install",&sysfip,par.echo))) 
           goto error;
    }              

    if(par.uninstall) 
    { if((sysfip.flags & E_SYSF_FLAG_U) && !(sysfip.flags & E_SYSF_FLAG_g) &&
         !execute_aux_com("-Remove","-uninstall",&sysfip,par.echo)) 
           goto error;
    }   

    if(par.info)  
    { E_NULL_ERROR(get_info(conf,&sysfip,par.echo));   
    }; 

    if(par.archive) 
    { E_NULL_ERROR(archive(conf,&sysfip,par.echo));  
    };  

    if(par.new)   
    { E_NULL_ERROR(new(conf,&sysfip,par.echo));        
    }; 

    if(par.makedef)   
    { E_NULL_ERROR(makedef(conf,&sysfip,par.echo));        
    }; 

    if(par.makeshared)   
    { E_NULL_ERROR(makedef(conf,&sysfip,par.echo));        
    }; 

    if(par.makegroup[0])   
    { E_NULL_ERROR(makedef(conf,&sysfip,par.echo));        
    }; 

    if(par.install_mode || par.install_uid || par.install_gid) 
    { E_NULL_ERROR(install_file_props(&sysfip,par.install_mode,
                   par.install_uid,par.install_gid,par.echo));  
    };  

    if(par.archive_mode || par.archive_uid || par.archive_gid) 
    { E_NULL_ERROR(archive_file_props(&sysfip,par.archive_mode,
                   par.archive_uid,par.archive_gid,par.echo)); 
    };  

    if(par.check) 
    { E_NULL_ERROR(check(conf,&sysfip,par.echo));      
    }; 

    if(par.diff)  
    { E_NULL_ERROR(diff(conf,&sysfip,par.echo));       
    }; 

    if(par.cat)   
    { E_NULL_ERROR(cat_config(conf,&sysfip,par.echo)); 
    }; 

    if(par.catsys)   
    { E_NULL_ERROR(cat_system(conf,&sysfip,par.echo)); 
    }; 
  }
end:
  return(1);

error:
  if(conf) e_error(E_ERROR_BAD_ARGUMENT,E_ERROR_LEVEL_USER,
            "ERROR : sysf aborted while processing %s on %s",conf,HostName);
error2:
  if(stp) E_FREE(stp);
  return(0);
}

/****************************************/
/* Return the sysf command name in name */
/* name : buffer                        */
/****************************************/
void get_command_name(char *name)
{ 
  if(!name) return;
  name[0] = 0;
  if(par.makedef) strcpy(name,"makedef");
  if(par.makeshared) strcpy(name,"makeshared");
  if(par.makegroup[0]) strcpy(name,"makegroup");
  if(par.new) strcpy(name,"new");
  if(par.uninstall) strcpy(name,"uninstall");
  if(par.install) strcpy(name,"install");
  if(par.first_install) strcpy(name,"first_install");
  if(par.OS_install) strcpy(name,"OS_install");
  if(par.info) strcpy(name,"info");
  if(par.archive) strcpy(name,"archive");
  if(par.check) strcpy(name,"check");
  if(par.is_not_conf) strcpy(name,"is_not_conf");
  if(par.diff) strcpy(name,"diff");
  if(par.check_host[0]) strcpy(name,"check_host");
  if(par.diff_host[0])  strcpy(name,"diff_host");
  if(par.cat) strcpy(name,"cat");
  if(par.catsys) strcpy(name,"catsys");

  if(par.install_mode || par.install_uid || par.install_gid) 
        strcpy(name,"install_mode/uid/gid");
  if(par.archive_mode || par.archive_uid || par.archive_gid) 
        strcpy(name,"archive_mode/uid/gid");

}

/**********************************************/
/* Check the number of sysf command selected  */
/* There should be only one.                  */
/* If there is none, info will be the default */
/* return : the number of comand selected     */
/**********************************************/
int commands_no()
{ int no_com;

  no_com = 0;
  if(par.makedef)      no_com++; 
  if(par.makeshared)   no_com++; 
  if(par.makegroup[0]) no_com++; 
  if(par.new)          no_com++; 
  if(par.uninstall)    no_com++;
  if(par.install)      no_com++;
  if(par.first_install) no_com++; 
  if(par.OS_install)   no_com++; 
  if(par.info)         no_com++;
  if(par.archive)      no_com++; 
  if(par.check)        no_com++;
  if(par.is_not_conf)  no_com++;
  if(par.diff)         no_com++;
  if(par.check_host[0]) no_com++; 
  if(par.diff_host[0]) no_com++; 
  if(par.cat)          no_com++; 
  if(par.catsys)       no_com++;
  if(par.list_groups)     no_com++;
  if(par.list_configs)    no_com++;
  if(par.list_1group[0])  no_com++;
  if(par.list_hosts)      no_com++;
  if(par.list_arg_hosts)  no_com++;
  if(par.list_arg_entry_names)  no_com++;
  if(par.list_host_group[0]) no_com++;
  if(par.list_these_hosts_groups)  no_com++;
  if(par.find_group)  no_com++;

  if(par.install_mode || par.install_uid || par.install_gid) no_com++;
  if(par.archive_mode || par.archive_uid || par.archive_gid) no_com++;

  return(no_com);
}

/*********************************************************/
/* Check if the command executed performe some changes : */
/* all install and archives commands                     */
/* Return : 1 : command changes some files               */ 
/*          0 commands runs a test and his harmless      */
/*********************************************************/
int is_a_modification_commands()
{ int no_com;

  /* only one command would have been selected so no_com is 1 or 0 */
  no_com = 0;
  if(par.makedef)      no_com++; 
  if(par.makeshared)   no_com++; 
  if(par.makegroup[0]) no_com++; 
  if(par.new)          no_com++; 
  if(par.uninstall)    no_com++;
  if(par.install)      no_com++;
  if(par.first_install) no_com++; 
  if(par.OS_install)   no_com++; 
  if(par.archive)      no_com++; 

  if(par.install_mode || par.install_uid || par.install_gid) no_com++;
  if(par.archive_mode || par.archive_uid || par.archive_gid) no_com++;

  return(no_com);
}
