#ifndef SYSF_COM_H
#define SYSF_COM_H
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

/* set of flags */
#define E_SYSF_FLAG_C 1
#define E_SYSF_FLAG_I (1<<1)
#define E_SYSF_FLAG_U (1<<2)
#define E_SYSF_FLAG_s (1<<3)
#define E_SYSF_FLAG_h (1<<4)
#define E_SYSF_FLAG_l (1<<5)
#define E_SYSF_FLAG_d (1<<6)
#define E_SYSF_FLAG_r (1<<7)
#define E_SYSF_FLAG_g (1<<8)
#define E_SYSF_FLAG_W (1<<9)

#define E_CONFIG_HOST    1
#define E_CONFIG_GROUP   2
#define E_CONFIG_DEFAULT 4
#define E_CONFIG_SHARED  8

#define E_SYSF_TYPE_SYMLINK  1
#define E_SYSF_TYPE_DIR      2

#define IS_SYMLINK(F) (F&E_SYSF_TYPE_SYMLINK)
#define IS_DIR(F)     (F&E_SYSF_TYPE_DIR)

#define INSTALL_MODE_FIRST_INSTALL 1

typedef struct {
  char FileName[E_PATH_SIZE];  /* File name              */
  char LinkPath[E_PATH_SIZE];  /* Path for Symbolic Link */
  time_t time;                 /* Last modification time */
  uid_t uid;                   /* File owner uid         */
  gid_t gid;                   /* File group             */
  mode_t mode;                 /* File access mode       */
  int size;                    /* File size              */
  int exists;                  /* 1 if the file exists   */
  int none;                    /* 1 if config file points to NONE */
  int type;                    /* symbolic link, dir (0 for reg. file) */
  int config_file_type;        /* HOST, GROUP or DEFAULT */
} file_info;

typedef struct {
  file_info sys_file;
  file_info config_file;
  file_info aux_com;
  int differ;
  int mode_differ;
  int sys_file_older;
  int flags;
  e_aset *dir_sys_asp;
  e_aset *dir_config_asp;
  e_aset *diff_asp;
} sysfile_info;

int get_info(char *name,sysfile_info *sip,int echo);
int check(char *name,sysfile_info *sip,int echo);
int is_not_conf(char *name,sysfile_info *sip,int echo);
int install(char *name,sysfile_info *sip,int echo);
int first_install(char *name,sysfile_info *sip,int echo);
int archive(char *name,sysfile_info *sip,int echo);
int new(char *name,sysfile_info *sip,int echo);
int makedef(char *name,sysfile_info *sip,int echo);
int diff(char *name,sysfile_info *sip,int echo);
int diff_host(char *name,file_info *fip,char *host1,sysfile_info *sip,
              char *host2,int echo);
int check_host(char *name,file_info *fip,char *host1,sysfile_info *sip,
               char *host2,int echo);
int cat_config(char *name,sysfile_info *sip,int echo);
int cat_system(char *name,sysfile_info *sip,int echo);
int install_file_props(sysfile_info *sip,int do_mode,int do_uid,int do_gid,
                       int echo);
int archive_file_props(sysfile_info *sip,int do_mode,int do_uid,int do_gid,
                       int echo);
int get_system_file(char *conf_name,e_aset *conf_data,file_info *fp,int flag);
int get_config_file(char *conf_name,char *conf_dir,e_aset *conf_data,
                    file_info *fp,char *host,int NoCheck,int flag);
int get_system_file_flags(char *conf_name,e_aset *conf_data,int *flags);
int get_auxiliary_command(char *conf_name,char *conf_dir,e_aset *conf_data,
                    file_info *fp);
int execute_aux_com(char *action, char *command,sysfile_info *sip,int echo);
int test_sysfile(sysfile_info *sip);
int files_are_equal(char *f1,char *f2);
int test_file_type(int flag, file_info *sfip, file_info *cfip, int mode);
int read_first_line(char *file,char *buff,int size);
int get_user_name(int uid,char *buff);
int get_group_name(int gid,char *buff);
int mk_path_dir(char *path,int echo);
int remove_dir(char *path,int echo);
int remove_file(char *path,int echo);
int rename_file(char *path,char *new_path,int echo);
int copy_file(char *path,char *new_path,int echo);
int make_sym_link(char *path,char *link_path,int echo);
int diff_files(char *path1,char *path2,char *name,int echo);
int md5sum_com(char *path,char *buff,int echo);

#endif
